#!/usr/bin/env python3

from discord.ext import commands

from dungeondice.lib import dice
from dungeondice.discord import templates


class Roll(commands.Cog):

    def __init__(self):
        self.diceparser = dice.Parser()

    @commands.command(
        name='roll',
        aliases=['r'],
        description='roll',
        brief='example: "2x2d20(poison)+d8(piercing)-4"',
        help='''Roll dice in a format like "2x2d20(poison)+d8(piercing)-4"

Rolls consist of multiple layers. The parser first cuts up rollstrings
into multiple 'groups' of 'sets' by parsing all the x and , modifiers.
The 'x' being a multiplier that creates multiple of the same rollgroups.
The ',' being a separator that allows you to create multiple different
groups in one go.
Everything behind the 'x' modifier is treated as part of the multiplier
until terminated by a ','.

Examples:
2xd20+d10:     Roll d20+d10 twice. Returning two different groups with
                their own totals.
d20,d20:       Roll a d20 twice. Returning two different groups with their
                own totals. In this case it being the total of 1 dice.
2xd20+d10,d10: Roll d20+d10 twice, roll d10 once. Returning three different
                groups with their own totals.
''',
    )
    async def roll(
        self,
        ctx,
        dicestring: dice.rollstring,
        *, message: str = ''
    ):
        """Uses the diceparser to roll dice."""
        await ctx.send(
            templates.dicerolls(
                ctx.author.display_name,
                self.diceparser.parse(dicestring),
                message
            )
        )

    async def cog_command_error(self, ctx, error):
        if isinstance(error, commands.BadArgument):
            await ctx.send('Invalid dicestring.')
        else:
            print(error)
            await ctx.send('Uncaught error while rolling.')



import pytest

from dungeondice.lib import dice


@pytest.mark.parametrize("rollstring,expected", [
    # Should pass
    ('1d20', [
        dice.Rollgroup.from_string('1d20'),
    ]),
    ('2x1d20', [
        dice.Rollgroup.from_string('1d20'),
        dice.Rollgroup.from_string('1d20'),
    ]),
    ('2xd20', [
        dice.Rollgroup.from_string('d20'),
        dice.Rollgroup.from_string('d20'),
    ]),
    ('2xd20(piercing)', [
        dice.Rollgroup.from_string('d20(piercing)'),
        dice.Rollgroup.from_string('d20(piercing)'),
    ]),
    ('2xd20,2d20', [
        dice.Rollgroup.from_string('d20'),
        dice.Rollgroup.from_string('d20'),
        dice.Rollgroup.from_string('2d20'),
    ]),
    ('2x2d20(poison)+d8(piercing)+d20', [
        dice.Rollgroup.from_string('2d20(poison)+d8(piercing)+d20'),
        dice.Rollgroup.from_string('2d20(poison)+d8(piercing)+d20'),
    ]),
    # Should fail
    ('1d', False),
    ('100d', False),
    ('5d10l6', False),
    ('d8(piercing))', False),
    ('d8(())piercing))', False),
    ('2xd20xd20', False),
    ('2xd202xd20', False),
])
def test_parser(rollstring, expected):
    """Test our rolling 'templating'.

    These tests onlt concern themselves with 'does this rollstring parse'.
    Rolling logic is tested in more specific tests.
    """
    parser = dice.Parser()

    if expected:
        assert parser.parse(rollstring) == expected
    else:
        with pytest.raises(ValueError):
            parser.parse(rollstring)


@pytest.mark.parametrize("rollstring,expected_result", [
    ('d20+5+d10', 9),
    ('d20-1+d10', 3),
    ('d20-1+2d10', 5),
    ('d20-1(piercing)+2d10(poison)', 5),
    ('d20-d10(poison)', 0),
])
def test_rollgroups(rollstring, expected_result):
    """Test creating and rolling rollgroups.

    We fumble rolls with a 2 to check totals.
    """
    rollgroup = dice.Rollgroup.from_string(rollstring)
    rollgroup.roll(fumble=2)
    assert rollgroup.total == expected_result


@pytest.mark.parametrize("rollstring,expected_rollset,expected_result", [
    ('d20', dice.Rollset('d20', 1, 20, 1, False), 2),
    ('2d20', dice.Rollset('2d20', 2, 20, 2, False), 4),
    ('2d20k1', dice.Rollset('2d20k1', 2, 20, 1, True), 2),
    ('2d20kl1', dice.Rollset('2d20kl1', 2, 20, 1, False), 2),
    ('2', dice.Rollset('2', 0, 0, 0, False), 2),
    ('2d20k1(poison)', dice.Rollset('2d20k1', 2, 20, 1, True), 2),
])
def test_rollsets(rollstring, expected_rollset, expected_result):
    """Test creating and rolling rollsets.

    We fumble rolls with a 2 to check totals.
    """
    rollset = dice.Rollset.from_string(rollstring)
    assert rollset == expected_rollset

    rollset.roll(fumble=2)
    assert rollset.total == expected_result


@pytest.mark.parametrize("rollstring,expected", [
    ('2d20k1(poison)', 'poison'),
])
def test_comments(rollstring, expected):
    """Test adding comments to a rollset."""
    rollset = dice.Rollset.from_string(rollstring)
    assert rollset.comment == expected



#!/usr/bin/env python3

import discord
from discord.ext import commands

import os
import sys
from dotenv import load_dotenv

from dungeondice.discord import commandgroups


load_dotenv()
TOKEN = os.getenv('DISCORD_TOKEN')

intents = discord.Intents.default()
intents.members = True
intents.message_content = True

bot = commands.Bot(command_prefix='!', intents=intents)


@bot.event
async def on_ready():
    if bot.user:
        print(f'Logged in as {bot.user} (ID: {bot.user.id})')
        print('------')
    else:
        print('Something is seriously wrong with this bot.')

    await bot.add_cog(commandgroups.Roll())


def start_bot():
    if TOKEN:
        bot.run(TOKEN)
    else:
        sys.exit("Missing a discord token. Supply it as environment variable.")
